use std::time::Instant;

pub fn execute_task01(content: &str) {
    let start = Instant::now();
    let product_correct_races = solve_01(content).unwrap();
    let duration = start.elapsed();

    assert_eq!(1624896, product_correct_races);
    println!("Day06 - Task01 - Duration: {duration:?} - Product Correct Races: {}", product_correct_races)
}

pub fn execute_task02(content: &str) {
    let start = Instant::now();
    let product_correct_races = solve_02(content).unwrap();
    let duration = start.elapsed();

    //assert_eq!(322500873, product_correct_races);
    println!("Day06 - Task02 - Duration: {duration:?} - Correct Race: {}", product_correct_races)
}

fn solve_01(content: &str) -> anyhow::Result<usize> {
    let result = content
        .lines()
        .map(|line| line
            .split_once(":")
            .unwrap()
            .1
            .trim())
        .map(|line| line
            .split(" ")
            .filter(|part| part.len() > 0)
            .map(|data| data.parse::<i64>().unwrap())
            .collect::<Vec<i64>>())
        .collect::<Vec<Vec<i64>>>();

    let mut races = vec![];

    for i in 0..result[0].len() {
        races.push((result[0][i].clone(), result[1][i].clone()))
    }

    Ok(races
        .iter()
        .map(|(time, distance)| calc_possible_options(*time, *distance))
        .map(|options| options.len())
        .product())
}

fn solve_02(content: &str) -> anyhow::Result<usize> {
    let result = content
        .lines()
        .map(|line| line
            .split_once(":")
            .unwrap()
            .1
            .trim())
        .map(|line| line
            .replace(" ", "")
            .parse::<i64>().unwrap())
        .collect::<Vec<i64>>();

    let mut races = vec![(result[0].clone(), result[1].clone())];
    Ok(races
        .iter()
        .map(|(time, distance)| calc_possible_options(*time, *distance))
        .map(|options| options.len())
        .product())
}


fn calc_possible_options(time: i64, distance: i64) -> Vec<i64> {
    let mut data = vec![];
    // TODO Performance!!!
    for i in 1..time {
        let y = i * (time - i);
        if y > distance {
            data.push(i)
        } else if data.len() > 0 {
            break
        }
    }

    data
}

#[test]
fn test_solve_01() {
    let example = r#"Time:      7  15   30
Distance:  9  40  200"#;

    let result = solve_01(example).unwrap();

    assert_eq!(288, result);
}

#[test]
fn test_solve_02() {
    let example = r#"Time:      7  15   30
Distance:  9  40  200"#;

    let result = solve_02(example).unwrap();

    assert_eq!(71503, result);
}
